//
//  SXTrack.h
//  SXVideoEnging
//
//  Created by Yin Xie on 2020/3/31.
//  Copyright © 2020 Zhiqiang Li. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <UIKit/UIKit.h>
#import "SXEffect.h"
#import "SXShape.h"
NS_ASSUME_NONNULL_BEGIN

typedef enum : NSUInteger {
    SXTrackTypeMedia            = 1 << 1,
    SXTrackTypeSticker          = 1 << 2,
    SXTrackTypeText             = 1 << 3,
} SXTrackType;

typedef enum : NSUInteger {
    SXBlendNormal    = 0,
    SXBlendLighten,
    SXBlendDarken,
    SXBlendMultiply,
    SXBlendAverage,
    SXBlendAdd,
    SXBlendSubtract,
    SXBlendDifference,
    SXBlendNegation,
    SXBlendExclusion,
    SXBlendScreen,
    SXBlendOverlay,
    SXBlendSoft_Light,
    SXBlendHard_Light,
    SXBlendColor_Dodge,
    SXBlendColor_Burn,
    SXBlendLinear_Dodge,
    SXBlendLinear_Burn,
    SXBlendLinear_Light,
    SXBlendVivid_Light,
    SXBlendPin_Light,
    SXBlendHard_Mix,
    SXBlendReflect,
    SXBlendGlow,
    SXBlendPhoenix,
//    SXBlendOpacity,
    SXBlendHue,
    SXBlendSaturation,
    SXBlendColor,
    SXBlendLuminosity,

    SXBlendNum_Modes,
} SXBlendMode;
@protocol SXFilterDelegate <NSObject>
/**
 * 添加滤镜，时间处理方式为伸缩
 * @param filterPath 资源路径
 * @return 滤镜对象，失败返回nullptr
*/
- (SXFilterEffect *)addFilter:(NSString *)filterPath;
/**
 * 通过复制已有的effect添加滤镜，时间处理方式为伸缩
 * @param effect 被拷贝的对象
 * @return 滤镜对象，失败返回nullptr
*/
- (SXFilterEffect *)addFilterByCopy:(SXFilterEffect *)effect;

/**
 * 移除滤镜
 * @param filterId 滤镜ID
*/
- (void)removeFilter:(NSString *)filterId;

/**
 * 获取当前对象所有应用的滤镜对象
 * @return 滤镜数组
*/
- (NSArray *)getFilters;

@end

@protocol SXAnimationDelegate <NSObject>
/**
 * 添加视频动画，时间处理方式为伸缩
 * @param animationPath 资源路径
 * @return 视频动画对象，失败返回nullptr
*/
- (SXAnimationEffect *)addAnimation:(NSString *)animationPath;
/**
 * 通过复制已有的effect添加视频动画，时间处理方式为伸缩
 * @param effect 被拷贝的对象
 * @return 视频动画对象，失败返回nullptr
*/
- (SXAnimationEffect *)addAnimationByCopy:(SXAnimationEffect *)effect;

/**
 * 移除视频动画
 * @param animationId 视频动画ID
*/
- (void)removeAnimation:(NSString *)animationId;

/**
 * 获取当前对象所有应用的视频动画对象
 * @return 视频动画数组
*/
- (NSArray *)getAnimations;
@end

@protocol SXVideoEffectDelegate <NSObject>
/**
 * 添加视频特效，时间处理方式为重复
 * @param videoEffectPath 资源路径
 * @return 视频特效对象，失败返回nullptr
*/
- (SXVideoEffect *)addVideoEffect:(NSString *)videoEffectPath;
/**
 * 通过复制已有的effect添加视频特效，时间处理方式为重复
 * @param effect 被拷贝的对象
 * @return 视频特效对象，失败返回nullptr
*/
- (SXVideoEffect *)addVideoEffectByCopy:(SXVideoEffect *)effect;

/**
 * 移除视频特效
 * @param videoEffectId 视频特效ID
*/
- (void)removeVideoEffect:(NSString *)videoEffectId;

/**
 * 获取当前对象所有应用的视频特效对象
 * @return 视频特效数组
*/
- (NSArray *)getVideoEffects;

@end

@protocol SXTrackAnimationDelegate <NSObject>
/**
 * 添加轨道动画
 * @note 动画时间线不能重叠
 * @note 入场/出场动画不能与循环动画共存
 *
 * @param trackAnimationPath  轨道动画资源包
 * @return 轨道动画对象t，失败返回nullptr
*/
- (SXTrackAnimationEffect *)addTrackAnimation:(NSString *)trackAnimationPath;

/**
 * 移除轨道动画
 * @param trackAnimationId 轨道动画ID
*/
- (void)removeTrackAnimation:(NSString *)trackAnimationId;

/**
 * 获取当前对象所有应用的轨道动画对象
 * @return 轨道动画数组
*/
- (NSArray *)getTrackAnimations;

@end

@protocol SXTextAnimationDelegate <NSObject>
/**
 * 添加文字动画
 * @param textAnimationPath 文字动画资源
 * @return 文字动画对象，失败时返回nullptr
*/
- (SXTextAnimationEffect *)addTextAnimation:(NSString *)textAnimationPath;

/**
 * 移除文字动画
 * @param textAnimationId 文字动画ID
*/
- (void)removeTextAnimation:(NSString *)textAnimationId;

/**
 * 获取当前对象所有应用的文字动画对象
 * @return 文字动画数组
*/
- (NSArray *)getTextAnimations;

@end

@protocol SXColorEffectDelegate <NSObject>
/**
 * 添加颜色调节
 * @return 颜色调节对象，失败返回nullptr（license不支持）
*/
- (SXColorAdjustEffect *)addColorAdjust;
/**
 * 通过复制已有的effect添加颜色调节
 * @return 颜色调节对象，失败返回nullptr（license不支持）
*/
- (SXColorAdjustEffect *)addColorAdjustByCopy:(SXColorAdjustEffect *)effect;

/**
 * 移除颜色调节
 * @param colorAdjustId 颜色调节ID
*/
- (void)removeColorAdjust:(NSString *)colorAdjustId;

/**
 * 获取当前对象所有应用的颜色调节对象
 * @return 颜色调节数组
*/
- (NSArray *)getColorAdjusts;

@end

@class SXEditManager;

#pragma mark - SXTrack

/**
 * 轨道，分为多媒体轨道，贴纸轨道和文字轨道
 * @warning 注意所有的对象对比是否相等时请使用id对比，引用可能会不同
*/
@interface SXTrack : NSObject
//当前应用的变化矩阵
@property (nonatomic, assign) CGAffineTransform transform;
//不透明度
@property (nonatomic, assign) float opacity;
//是否纵向翻转
@property (nonatomic, assign) BOOL verticalFlip;
//是否横向翻转
@property (nonatomic, assign) BOOL horizontalFlip;

/**
 * 获取当前track的类型
 * @return 类型
*/
- (SXTrackType)type;

/**
 * 获取当前track是否可用
 * @return YES可用 NO不可用
*/
- (BOOL)isValid;

/**
 * 获取当前track的ID
 * @return ID
*/
- (NSString *)getTrackId;

/**
 * 调整track的位置，会改变track的transform
 * @param coverAll 是否填充满整个画面
 * @param margin 当coverAll为NO时，track在整个画面留边边距为margin的范围里适配宽高完整展示
*/
- (void)fitToEditContext:(BOOL)coverAll margin:(int)margin;

/**
 * 获取当前track的尺寸
 * @return size
*/
- (CGSize)getTrackSize;

/**
 * start time    in point                 out point    end time
 *    |             |                                  |               |
 *              Display part
 *    |             |-----------------------------|              |
 *                  |
 *                duration
 *    |-----------|-----------------------------|------------|
 *                  |
 *                 track
 * --------------------------------------------------------------------
 * 获取当前track的起点时间
 * @return 起点时间
*/
- (double)startTime;

/**
 * 获取当前track的展示时长
 * @return 展示时长
*/
- (double)duration;

/**
 * 获取当前track的插入时间
 * @return 插入时间
*/
- (double)inPoint;

@end

#pragma mark - SXMediaTrack
@interface SXMediaTrack : SXTrack<SXFilterDelegate, SXAnimationDelegate, SXVideoEffectDelegate, SXColorEffectDelegate>

//颜色混合模式
@property (nonatomic, assign) SXBlendMode blendMode;

/**
 * 获取媒体文件路径
 * @return 文件路径
*/
- (NSString *)getFilePath;

/**
 * 是否是静态图片
 * @return 是静态图片返回true，否则返回false
*/
- (BOOL)isStaticImage;

/**
 * track中的资源时长
 * @return 资源时长
*/
- (double)resourceDuration;

/**
 * 设置视频素材中的音频音量
 * @param volume 音量大小
*/
- (void)setVolume:(float)volume;

/**
 * 获取视频素材中的音频音量
 * @return 音量大小
*/
- (float)getVolume;

/**
 * 设置蒙版形状
 * @param shapes 蒙版形状的数组
*/
- (void)setMaskShape:(NSArray *)shapes;

/**
 * 是否需要反转蒙版
 * @param inverse YES 反转 NO 不反转
*/
- (void)setInverseMaskShape:(BOOL)inverse;

/**
 * 当前是否反转了蒙版
 * @return 是否反转
*/
- (BOOL)isInverseMaskShape;

/**
 * 获取当前蒙版的形状
 * @return 形状数组
*/
- (NSArray *)getMaskShape;

/**
 * 裁剪素材
 * @param size 裁剪的大小
 * @param transform 裁剪应用的变化，以左上角为锚点
*/
- (void)cropMediaWithSize:(CGSize)size transform:(CGAffineTransform)transform;

@end

#pragma mark - SXStickerTrack

@interface SXStickerTrack : SXTrack<SXTrackAnimationDelegate>

/**
 * track中的资源时长
 * @return 资源时长
*/
- (double)resourceDuration;

@end

#pragma mark - SXTextTrack

typedef enum : NSUInteger {
    SXStrokeStyleStrokeOverFill,
    SXStrokeStyleFillOverStroke,
} SXStrokeStyle;

typedef enum : NSUInteger {
    SXFontCacheModeImmediately,//不缓存字体，用完即释放
    SXFontCacheModeCache,//缓存字体，可能占用较告内存
} SXFontCacheMode;

typedef enum : NSUInteger {
    SXTextTypeLine,
    SXTextTypeBox,
} SXTextType;

typedef enum : NSUInteger {
    SXTextDirectionTypeHorizontal,
    SXTextDirectionTypeVertical,
} SXTextDirectionType;

typedef enum : NSUInteger {
    SXTextAlignmentTypeLeft,
    SXTextAlignmentTypeRight,
    SXTextAlignmentTypeCenter,
} SXTextAlignmentType;

@interface SXTextTrack : SXTrack<SXTextAnimationDelegate>
//设置文字内容
@property (nonatomic, strong) NSString *content;
//设置描边宽度，默认为0
@property (nonatomic, assign) float strokeWidth;
//设置文字大小，单位为像素
@property (nonatomic, assign) float fontSize;
//设置字符间距，单位为像素
@property (nonatomic, assign) int tracking;
//设置字体文件，默认使用初始设置的默认字体文件
@property (nonatomic, strong) NSString *fontFile;
//设置描边绘制方式，默认描边画在文字之下
@property (nonatomic, assign) SXStrokeStyle strokeStyle;
//设置文字方向
@property (nonatomic, assign) SXTextDirectionType direction;
//设置排版方式
@property (nonatomic, assign) SXTextAlignmentType alignment;

/**
 * 设置填充颜色，默认为白色
 * @param red [0-1]
 * @param green [0-1]
 * @param blue [0-1]
*/
- (void)setFillColorRed:(CGFloat)red green:(CGFloat)green blue:(CGFloat)blue;

/**
 * 设置描边颜色， 默认为白色
 * @param red [0-1]
 * @param green [0-1]
 * @param blue [0-1]
*/
- (void)setStrokeColorRed:(CGFloat)red green:(CGFloat)green blue:(CGFloat)blue;

/**
 * 设置文字气泡
 * @param resourcePath 文字气泡资源路径
 * @param error 错误码
 * @return 设置成功返回true，否则返回false
 */
- (BOOL)setBubble:(NSString *)resourcePath error:(int *)error;

/**
 * 移除文字气泡
 */
- (void)removeBubble;

/**
 * 气泡资源路径
 * @return 气泡资源路径，无气泡资源时，不可用
 */
- (NSString *)bubbleResourcePath;

/**
  * 设置预设样式
  * @param resourcePath 预设样式资源路径
  * @param error 错误码
  * @return 设置成功返回true，否则返回false
  */
- (BOOL)setTextStyle:(NSString *)resourcePath error:(int *)error;

/**
 * 重置文字样式为默认值
 */
- (void)resetTextStyle;

/**
 * 样式资源路径
 * @return 样式资源路径，无样式资源时，不可用
 */
- (NSString *)styleResourcePath;
@end

NS_ASSUME_NONNULL_END
